
import command

class ExecuteCommand(command.Command):
	"""Execute a command, given its name.
	"""
	name = 'execute-command'
	context = 'global'
	def call(self, con):
		complet = command.getcompletor()
		cmdname = inputstring.input_string_arg(
			self, 
			1,
			'Command', 
			self.name,
			complet)
		self.runcmdfree(cmdname)

class ShowCommandContexts(command.Command):
	"""Show the command contexts of the current focus.
	"""
	name = 'show-command-contexts'
	context = 'focusable'
	def call(self, con):
		ls = self.executor.getbindcontexts()
		logo = log.Logger('*contexts*', 1)
		for obj in ls:
			st = str(obj.__class__)
			### have a commandcontext label property?
			if (isinstance(obj, buffer.Buffer)):
				st = st + ' (' + obj.name + ')'
			if (isinstance(obj, window.Window)):
				st = st + ' (' + obj.data.name + ')'
			logo.write(st + ':\n')
			for cla in obj.getcommandcontexts():
				st = '  ' + execute.contexts[cla] + '\n'
				logo.write(st)

class ImportModule(command.Command):
	"""Import a Python module.
	"""
	name = 'import-module'
	context = 'global'
	def call(self, con):
		st = inputstring.input_string_arg(self, 1, 'Module', self.name)
		user.importmod(st)
		status.post('Imported '+st+'.')
				
class EvalExpression(command.Command):
	"""Evaluate an arbitrary Python expression, and display the result
	in the status line.
	"""
	name = 'eval-expression'
	context = 'global'
	def call(self, con):
		st = inputstring.input_string_arg(self, 1, 'Expression', 'pyexpr')
		val = user.eval(st)
		status.post(repr(val))
				
class EvalStatement(command.Command):
	"""Evaluate an arbitrary Python statement.
	"""
	name = 'eval-statement'
	context = 'global'
	def call(self, con):
		st = inputstring.input_string_arg(self, 1, 'Statement', 'pystatement')
		user.execstr(st)
				
class EvalExpressionLog(command.Command):
	"""Evaluate an arbitrary Python expression, and display the result
	in a text buffer.
	"""
	name = 'eval-expression-log'
	context = 'global'
	def call(self, con):
		st = inputstring.input_string_arg(self, 1, 'Expression', 'pyexpr')
		val = user.eval(st)
		log.log(repr(val)+'\n')
### select a log buffer (default to *log*)
				
command.register([
	ExecuteCommand, ShowCommandContexts, 
	ImportModule, EvalStatement, EvalExpression, EvalExpressionLog
])

command.bind('execute-command', 'esc x')
command.bind('show-command-contexts', 'esc c') ###
command.bind('eval-statement', 'esc ^e') ###
command.bind('eval-expression', 'esc e') ###
command.bind('eval-expression-log', 'esc E') ###
command.bind('import-module', 'esc i')

# late imports
import log
import inputstring
import window
import buffer
import execute
import status
import user
