import os
import signal

import command
	
class NoOp(command.Command):
	"""Do nothing.
	"""
	name = "noop"
	def call(self, con):
		pass

class Cancel(command.Command):
	"""Cancel an operation in progress.
	"""
	name = "cancel"
	def call(self, con):
		raise command.CancelException

class SuspendUmics(command.Command):
	"""Suspend this Umics process.
	"""
	name = "suspend-umics"
	def call(self, con):
		os.kill(os.getpid(), signal.SIGTSTP)

class ResizeScreen(command.Command):
	"""Adjust the screen layout to a new terminal size.
	"""
	name = "resize-screen"
	def call(self, con):
		status.resize(screen.stdscr)
		window.resize(screen.stdscr)

class RefreshScreen(command.Command):
	"""Redraw everything on the screen.
	"""
	name = "refresh-screen"
	def call(self, con):
		status.redrawwin(screen.stdscr)
		window.redrawwin(screen.stdscr)

command.register([
	NoOp, Cancel, SuspendUmics, ResizeScreen, RefreshScreen
])

command.bind(SuspendUmics, '^Z')
command.bind(Cancel, '^G')
command.bind(ResizeScreen, 'resize')
command.bind(RefreshScreen, '^L')
command.bind(NoOp, 'error')

# late imports
import status
import window
import screen
import log

