import os.path

# interactive commands

import command

class QuitUmics(command.Command):
	"""Kill this Umics process.
	"""
	name = 'quit-umics'
	context = 'window'
	def call(self, con):
		raise command.QuitException

class WindowClose(command.Command):
	"""Close the current window.
	"""
	name = 'window-close'
	context = 'window'
	def call(self, win):
		window.close(win)

class WindowCloseOthers(command.Command):
	"""Close all windows except the current window.
	"""
	name = 'window-close-others'
	context = 'window'
	def call(self, win):
		others = filter((lambda w: w != win), window.windows)
		for w in others:
			window.close(w)

class WindowSplit(command.Command):
	"""Split the current window.
	"""
	name = 'window-split'
	context = 'window'
	def call(self, win):
		window.open(win.data)

class WindowChangeBuffer(command.Command):
	"""Change the current window to view a different buffer.
	"""
	name = 'window-change-buffer'
	context = 'window'
	def call(self, win):
		complet = buffer.getcompletor()
		complet.set_needs_exact(1)
		bufname = inputstring.input_string_arg(
			self, 
			1,
			'Switch to buffer', 
			self.name,
			complet)
		if (bufname == ''):
			return
		buf = buffer.findbyname(bufname)
		if (buf == None):
			### create buffer? what type?
			raise command.StatusBeepWarning('No such buffer: '+bufname)
		win.checkvalid()
		if (win.data == buf):
			return
		window.reopen(win, buf)

class WindowKillBuffer(command.Command):
	"""Close a buffer.
	"""
	name = 'window-kill-buffer'
	context = 'window'
	def call(self, win):
		defst = win.data.name
		complet = buffer.getcompletor()
		complet.set_needs_exact(1)
		complet.set_extra_matches('')
		bufname = inputstring.input_string_arg(
			self, 
			1,
			'Kill buffer ('+defst+')', 
			self.name,
			complet)
		if (bufname == ''):
			bufname = defst
		buf = buffer.findbyname(bufname)
		if (buf == None):
			raise command.StatusBeepWarning('No such buffer: '+bufname)
		### make buffer command
		winlist = filter(
			lambda win: (win.data == buf),
			window.windows)
		for win in winlist:
			window.close(win)
		buf.kill()

class WindowRotateFocus(command.Command):
	"""Change focus to the next window.
	"""
	name = 'window-focus-next'
	context = 'focusable'
	def call(self, con):
		ix = window.focuswin+1
		if (ix >= len(window.windows)):
			if (minibuf.active):
				ix = -1
			else:
				ix = 0
		window.changefocus(ix)

class WindowNewBuffer(command.Command):
	"""Open a new empty buffer of a given type.
	"""
	name = 'window-new-buffer'
	context = 'global'
	def call(self, con):
		bufnam = inputstring.input_string_arg(
			self, 
			1,
			'New buffer name', 
			self.name)
		if (bufnam == ''):
			raise command.StatusBeepWarning('Buffer name required.')
		if (buffer.findbyname(bufnam)):
			raise command.StatusBeepWarning('Buffer already exists: '+bufnam)
		complet = buffer.getclasscompletor()
		complet.set_needs_exact(1)
		buftyp = inputstring.input_string_arg(
			self, 
			2,
			'New buffer type', 
			self.name+'.class',
			complet)
		bufcla = buffer.bufferclasses[buftyp]
		buf = bufcla(bufnam)
		window.open(buf)

command.register([
	QuitUmics, 
	WindowClose, WindowCloseOthers, WindowSplit, 
	WindowRotateFocus,
	WindowNewBuffer,
	WindowChangeBuffer, WindowKillBuffer
])

command.bind('quit-umics', '^X ^C')
command.bind('window-focus-next', '^X o')

command.bind('window-close', '^X 0')
command.bind('window-close-others', '^X 1')
command.bind('window-split', '^X 2')
command.bind('window-change-buffer', '^X b')
command.bind('window-kill-buffer', '^X k')


# late imports
import log
import focus
import window
import buffer
import status
import minibuf
import inputstring
import inputkey
import watcher
