import string
import curses

import text

class QuitException(Exception):
	pass
class CancelException(Exception):
	pass
class DoneException(Exception):
	pass
class AbortException(Exception):
	pass
class StatusWarning(Exception):
	pass
class StatusBeepWarning(StatusWarning):
	pass

class Command:
	context = None
	executor = None
	args = ()

	def __init__(self, exex, args):
		self.executor = exex
		self.args = args

	def getarg(self, argnum=1):
		if (len(self.args) <= argnum-1):
			return None ### interactive fetch?
		return self.args[argnum-1]

	def countargs(self):
		return len(self.args)

	def call(self):
		raise Exception, 'Command lacks call property'

	def runcmdfree(self, cmdname, *args):
		res = commands[cmdname]
		exex = execute.ExecutionContext(self.executor)
		ls = exex.getbindcontexts()
		for obj in ls:
			if (isinstance(obj, res.context)):
				break
		else:
			raise StatusBeepWarning(cmdname+' is not in the current contexts.')
		cmd = res(exex, args)
		cmd.call(obj)

	def runcmd(self, cmdname, obj, *args):
		res = commands[cmdname]
		exex = execute.ExecutionContext(self.executor)
		if (not isinstance(obj, res.context)):
			raise StatusBeepWarning('Wrong object context for '+cmdname+'.')
		cmd = res(exex, args)
		cmd.call(obj)

class CommandIncomplete(Command):
	name = intern('incomplete-command')

bindings = {}
	
commands = {}

def addtwo(map, key, subkey, value):
	submap = map.get(key)
	if (submap == None):
		submap = {}
		map[key] = submap
	submap[subkey] = value

def gettwo(map, key, subkey):
	submap = map.get(key)
	if (submap == None):
		return submap
	return submap.get(subkey)

cursesequiv = {
	'UP':None,
	'DOWN':None,
	'LEFT':None,
	'RIGHT':None,
	'RESIZE':None,
	'BACKSPACE':None,
	'DC':'delete',
	'IC':'insert',
	'NPAGE':('pagedown','next'),
	'PPAGE':('pageup','prior'),
}

keynames = {
	'space' : ord(' '),
	'spc' : ord(' '),
	'esc' : 27,
	'escape' : 27,  # ctrl-[
	'tab' : 9,      # ctrl-I
	'return' : 13,  # ctrl-M
	'error' : -1,
}

inversekeynames = {
	ord(' ') : 'space',
	27 : 'esc',
}

KEY_UNAVAILABLE = ()

def preparekeynames():
	for ckey in cursesequiv.keys():
		val = cursesequiv[ckey]
		if (val == None):
			val = (ckey.lower(),)
		if (type(val) == str):
			val = (val,)
		key = getattr(curses, 'KEY_'+ckey, KEY_UNAVAILABLE)
		for nam in val:
			keynames[nam] = key
		if (key != KEY_UNAVAILABLE):
			inversekeynames[key] = val[0]
			
	ix = 1
	while 1:
		key = getattr(curses, 'KEY_F'+str(ix), None)
		if (key == None or key <= 255):
			break
		keynames['f'+str(ix)] = key
		inversekeynames[key] = 'f'+str(ix)
		ix = ix+1
	
preparekeynames()

def register(ls):
	if (type(ls) != tuple and type(ls) != list):
		ls = [ls]
	for cla in ls:
		if not issubclass(cla, Command):
			raise TypeError, 'register(): expects Command or sequence of Command'
		cla.name = intern(cla.name)

		comx = cla.context
		if (comx == None):
			comx = execute.GlobalCommandContext
		if (type(comx) == str):
			comx = execute.getcontextbyname(comx)
		if not execute.isregistered(comx):
			raise ValueError, 'register(): Command has invalid context: '+cla.name

		cla.context = comx
		commands[cla.name] = cla

def bind(cla, keystr, *cmdargs):
	if (type(cla) == str):
		nam = intern(cla)
	else:
		assert (issubclass(cla, Command))
		nam = cla.name
	cmd = commands.get(nam)
	if (cmd == None):
		raise IndexError, ('bind(): undefined name "'+nam+'"')

	comx = cmd.context

	keylist = keystr.split(' ')
	if (len(keylist) == 0):
		raise ValueError, ('bind(): empty keylist for "'+nam+'"')

	ls = []
	for key in keylist:
		isctrl = 0
		ismeta = 0
		while (1):
			if (key[0] == '^' and len(key) > 1):
				isctrl = 1
				key = key[1:]
			elif (key[0:2] == 'C-' and len(key) > 2):
				isctrl = 1
				key = key[2:]
			elif (key[0:2] == 'M-' and len(key) > 2):
				ismeta = 1
				key = key[2:]
			else:
				break

		if (len(key)==1):
			val = ord(key)
		else:
			val = keynames.get(key.lower())
			if (val == KEY_UNAVAILABLE):
				# the curses library can't generate this key, so we should
				# just ignore this binding.
				return
			if (val == None):
				raise ValueError, ('bind(): cannot translate key "'+key+'" for "'+nam+'"')

		if (isctrl):
			if (val >= ord('@') and val <= ord('_')):
				val = val - ord('@')
			elif (val >= ord('a') and val <= ord('z')):
				val = val - ord('a') + 1
			elif (val == ord('?')):
				val = 127
			elif (val == ord(' ')):
				val = 0
		if (ismeta):
			ls.append(27)
		ls.append(val)

	ls = tuple(ls)
	for ix in range(len(ls)-1):
		### check for cmd/incomplete conflicts
		### could also have the magic partial-command prompt override
		addtwo(bindings, comx, ls[ : ix+1 ], (CommandIncomplete, ()))
	addtwo(bindings, comx, ls, (cmd, cmdargs))

def contextstobindings(conlist):
	bd = []
	for obj in conlist:
		for cla in obj.getcommandcontexts():
			bdo = bindings.get(cla)
			if (bdo != None):
				bd.append( (bdo, obj) )
	return bd
	
def getcompletor():
	arr = commands.keys()
	com = completion.FixedSetCompletor(arr)
	com.set_needs_exact(1)
	return com

def keyname(key):
	if inversekeynames.has_key(key):
		return inversekeynames[key]
	if (key >= 0 and key <= 255):
		return text.displaychar[key]
	return '[' + str(key) + ':' + curses.keyname(key) + ']'

def acceptkey(bindlist, keylist):
	for (bd, con) in bindlist:
		if bd.has_key(keylist):
			(res, resargs) = bd.get(keylist)
			return (res, con, resargs)
	return (None, None, ())

printablerange = map(
	lambda key: (key, keyname(key)),
	range(32, 127))
	
typeablerange = map(
	lambda key: (key, keyname(key)),
	range(0, 128))
	
# late imports
import execute
import status
import buffer
import log
import window
import completion

# import various input methods
import inputkey
import inputstring
