import string

import focus

class Minibuf(focus.Focusable):
	def __init__(self, prompt, history):
		self.watch = watcher.Watchable(self)
		self.body = ''
		self.mark = 0
		self.prompt = prompt
		self.completor = None
		self.history = history
		if (history == None):
			self.historypos = 0
		else:
			self.historypos = len(history)

	def getcommandcontexts(self):
		return [Minibuf] + focus.Focusable.getcommandcontexts(self)

	def replace(self, st, oldstart=0, oldend=None):
		if (oldstart < 0):
			oldstart = 0
		if (oldstart > len(self.body)):
			oldstart = len(self.body)
		if (oldend == None):
			oldend = len(self.body)
		if (oldend < oldstart):
			oldend = oldstart
		if (oldend > len(self.body)):
			oldend = len(self.body)
		self.body = self.body[:oldstart] + st + self.body[oldend:]

		if (self.mark >= oldstart and self.mark < oldend):
			self.mark = oldstart + len(st)
		elif (self.mark >= oldend):
			self.mark = self.mark + len(st) - (oldend-oldstart)
		
		self.watch.signal(watcher.SIG_DATA) ###

	def setmark(self, pos):
		if (pos < 0):
			pos = 0
		if (pos > len(self.body)):
			pos = len(self.body)
		if (pos == self.mark):
			return
		self.mark = pos
		self.watch.signal(watcher.SIG_DATA) ###

class MinibufCompletor(Minibuf):
	def __init__(self, prompt, history, completor):
		Minibuf.__init__(self, prompt, history)
		self.completor = completor
		
	def getcommandcontexts(self):
		return [MinibufCompletor] + Minibuf.getcommandcontexts(self)

def activate(prompt='', history=None, completor=None, initval=''):
	global buf, active
	if (active):
		raise command.StatusBeepWarning('Command attempted to use minibuffer while already active')
	active = 1
	if (completor == None):
		buf = Minibuf(prompt, history)
	else:
		buf = MinibufCompletor(prompt, history, completor)
	if (type(initval) == tuple):
		buf.body = initval[0]
		buf.mark = initval[1]
	else:
		buf.body = initval
		buf.mark = len(initval)
	buf.watch.link(status.statusline, status.statusline.see_change_minibuf)
	buf.watch.signal(watcher.SIG_DATA)

def deactivate():
	global buf, active
	if (not active):
		raise command.StatusBeepWarning('Command attempted to deactivate minibuffer while not active')
	buf.watch.signal(watcher.SIG_DATA)
	buf.watch.unlink(status.statusline)
	buf = None
	active = 0


# late imports
import command
import watcher
import execute
import status

active = 0
buf = None

execute.registercontext(Minibuf, 'minibuf')
execute.registercontext(MinibufCompletor, 'minibuf-completor')

# interactive commands

class MinibufInsertChar(command.Command):
	"""Insert a keystroke.
	"""
	name = "minibuf-insert-char"
	context = 'minibuf'
	def call(self, con):
		arg = self.getarg()
		con.replace(arg, con.mark, con.mark)

class MinibufInsertLiteral(command.Command):
	"""Insert a literal keystroke.
	"""
	name = "minibuf-insert-literal"
	context = 'minibuf'
	def call(self, con):
		arg = inputkey.input_ascii_arg(self, 1)
		con.replace(chr(arg), con.mark, con.mark)

class MinibufDeleteChar(command.Command):
	"""Delete the character at mark.
	"""
	name = "minibuf-delete-char"
	context = 'minibuf'
	def call(self, con):
		if (con.mark <= 0):
			raise command.StatusBeepWarning('Beginning of buffer')
		con.replace('', con.mark-1, con.mark)

class MinibufDeleteNextChar(command.Command):
	"""Delete the character after mark.
	"""
	name = "minibuf-delete-next-char"
	context = 'minibuf'
	def call(self, con):
		if (con.mark >= len(con.body)):
			raise command.StatusBeepWarning('End of buffer')
		con.replace('', con.mark, con.mark+1)

class MinibufKillToEnd(command.Command):
	"""Delete the rest of the line after mark.
	"""
	name = "minibuf-kill-to-end"
	context = 'minibuf'
	def call(self, con):
		if (con.mark >= len(con.body)):
			raise command.StatusBeepWarning('End of buffer')
		con.replace('', con.mark, len(con.body))

class MinibufBeginning(command.Command):
	"""Move mark to beginning of line.
	"""
	name = "minibuf-beginning"
	context = 'minibuf'
	def call(self, con):
		con.setmark(0)

class MinibufEnd(command.Command):
	"""Move mark to end of line.
	"""
	name = "minibuf-end"
	context = 'minibuf'
	def call(self, con):
		con.setmark(len(con.body))

class MinibufBack(command.Command):
	"""Move mark back one character.
	"""
	name = "minibuf-back"
	context = 'minibuf'
	def call(self, con):
		con.setmark(con.mark-1)

class MinibufForward(command.Command):
	"""Move mark forward one character.
	"""
	name = "minibuf-forward"
	context = 'minibuf'
	def call(self, con):
		con.setmark(con.mark+1)

class MinibufTranspose(command.Command):
	"""Transpose the character at mark with previous character.
	"""
	name = "minibuf-transpose"
	context = 'minibuf'
	def call(self, con):
		ix = con.mark
		if (ix >= len(con.body)):
			ix = ix-1
		if (ix <= 0):
			raise command.StatusBeepWarning('Beginning of buffer')
		st = con.body[ix] + con.body[ix-1]
		con.replace(st, ix-1, ix+1)

class MinibufHistoryBack(command.Command):
	"""Change the contents of the minibuffer to the previous history entry.
	"""
	name = "minibuf-history-back"
	context = 'minibuf'
	def call(self, con):
		if (con.history == None):
			raise command.StatusBeepWarning('No history available')
		if (con.historypos <= 0):
			raise command.StatusBeepWarning('Beginning of history')
		con.historypos = con.historypos-1
		st = con.history[con.historypos]
		con.replace(st, 0, len(con.body))
		
class MinibufHistoryForward(command.Command):
	"""Change the contents of the minibuffer to the next history entry.
	"""
	name = "minibuf-history-forward"
	context = 'minibuf'
	def call(self, con):
		if (con.history == None):
			raise command.StatusBeepWarning('No history available')
		if (con.historypos >= len(con.history)):
			raise command.StatusBeepWarning('End of history')
		con.historypos = con.historypos+1
		if (con.historypos >= len(con.history)):
			st = ''
		else:
			st = con.history[con.historypos]
		con.replace(st, 0, len(con.body))
		
class MinibufComplete(command.Command):
	"""Complete the contents of the minibuffer, as far as possible.
	"""
	name = "minibuf-complete"
	context = 'minibuf-completor'
	def call(self, con):
		assert (con.completor != None)
		(text, exact) = con.completor.completion(con.body)
		if (text != con.body):
			con.replace(text, 0, len(con.body))
		con.setmark(len(con.body))

class MinibufShowCompletions(command.Command):
	"""Show the possible completions for the contents of the minibuffer.
	"""
	name = "minibuf-show-completions"
	context = 'minibuf-completor'
	def call(self, con):
		assert (con.completor != None)
		ls = con.completor.possibilities(con.body)
		if (not len(ls)):
			raise command.StatusBeepWarning('No completions')
		ls.sort()
		buf = selbuffer.getbyname('*completions*')
		buf.changeprompt()
		buf.replace(ls)
		buf.ensureopen()

class MinibufAccept(command.Command):
	"""Accept the contents of the minibuffer.
	"""
	name = "minibuf-accept"
	context = 'minibuf'
	def call(self, con):
		if (con.completor != None and con.completor.needs_exact):
			if not(con.completor.match(con.body)
					or (con.body in con.completor.extra_matches)):
				raise command.StatusBeepWarning(None)
		raise command.DoneException

command.register([
	MinibufInsertChar, MinibufInsertLiteral,
	MinibufDeleteChar, MinibufDeleteNextChar, MinibufKillToEnd,
	MinibufBeginning, MinibufEnd, MinibufBack, MinibufForward,
	MinibufHistoryBack, MinibufHistoryForward,
	MinibufTranspose,
	MinibufComplete, MinibufShowCompletions,
	MinibufAccept
])

command.bind("minibuf-beginning", '^A')
command.bind("minibuf-end", '^E')
command.bind("minibuf-back", '^B')
command.bind("minibuf-back", 'left')
command.bind("minibuf-forward", '^F')
command.bind("minibuf-forward", 'right')
command.bind("minibuf-transpose", '^T')
command.bind("minibuf-delete-char", '^?')
command.bind("minibuf-delete-char", 'delete')
command.bind("minibuf-delete-next-char", '^D')
command.bind("minibuf-kill-to-end", '^K')
command.bind("minibuf-insert-literal", '^Q')
command.bind("minibuf-history-back", '^P')
command.bind("minibuf-history-back", 'up')
command.bind("minibuf-history-forward", '^N')
command.bind("minibuf-history-forward", 'down')
command.bind("minibuf-back", 'left')
command.bind("minibuf-complete", 'tab')
command.bind("minibuf-complete", 'space')
command.bind("minibuf-show-completions", '?')
command.bind("minibuf-accept", 'return')

for (key, keyname) in command.printablerange:
	command.bind("minibuf-insert-char", keyname, chr(key))


# late imports
import log
import inputkey
import selbuffer
